#ifndef KDK_DISK_ENCRYPT_H
#define KDK_DISK_ENCRYPT_H
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

typedef enum _crypt_disk_format
{
    DISK_FORMAT = 0,
    DISK_NO_FORMAT = 1,
} crypt_disk_format;

typedef enum
{
    KY_SUCCESS = 0,                         // 成功
    KY_ERROR = 100,                         // 普通操作流程失败
    KY_UMOUNT_FAIL = 101,                   // 卸载失败
    KY_INIT_ERROR = 102,                    // 初始化设备结构体失败
    KY_INVALID_DEVICE = 103,                // 不合法的设备
    KY_DECRYPT_BACKUP_HEADER = 104,         // 解密过程备份头过程失败
    KY_DECRYPT_ACTIVE_FAIL = 105,           // 解密过程映射头失败
    KY_DECRYPT_COPY_DATA = 106,             // 解密过程拷贝数据失败
    KY_FORMAT_FAIL = 107,                   // 加密过程的格式化失败
    KY_ADD_KEY_FAIL = 108,                  // 添加密码失败
    KY_ACTIVATE_FAIL = 109,                 // 映射(激活)设备失败
    KY_GET_DEVICE_TYPE_FAIL = 110,          // 获取设备类型失败
    KY_REENCRYPT_INIT_FAIL = 111,           // 执行crypt_reencryt_init*失败
    KY_OPEN_FILE_FAIL = 112,                // 打开文件失败
    KY_KEYSLOT_DESTROY_FAIL = 113,          // 销毁密钥槽失败
    KY_KEYSLOT_NOT_HAVE_BACKUPKEY = 114,    // 密钥槽未存在备份密钥
    KY_UUID_NOT_MATCH = 115,                // 备份密钥的UUID与设备UUID不匹配
    KY_CHANGE_PASSPHRASE_FAIL = 116,        // 修改密码失败
    KY_DEACTIVATE_FAIL = 117,               // 取消映射(激活)设备失败
    KY_PASSPHRASE_ERROR = 118,              // 密码错误
    KY_COMMAND_ERROR = 119,                 // 命令错误
    KY_DEVICE_SPACE_TOO_SMALL = 120,        // 硬盘分区大小过小
    KY_REENCRYPT_EXCEPTION_INTERRUPT = 121, // 保留数据加密异常中断
    KY_BACKUPKEY_EXPORT_FAIL = 122,         // 导出备份密钥失败
    KY_DBUS_FORBIDDEN = 123,		    // DBUS接口阻止
} KY_ERROR_LIST;

#ifdef __cplusplus
extern "C"
{
#endif
    /* 
启用磁盘加密  使未加密的磁盘变成加密密文磁盘
    输入参数
                device: 设备分区例如：/dev/sdb1
                format: 是否格式化
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_encrypt_device(char *device, crypt_disk_format format, char *password, size_t passwordLen, char *path);

    /* 
停用磁盘加密，使已经加密的磁盘变成未加密的明文磁盘
    输入参数
                device: 设备分区例如：分区uuid或者/dev/sdb1
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_decrypt_device(char *device, char *password, size_t passwortlen);

    /* 
解锁未打开的加密磁盘，使加密磁盘能够进行读写
    输入参数
                device: 设备分区例如：/dev/sdb1
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_open_encrypted_device(char *device, char *password, size_t passwortlen);

    /* 
锁定打开的加密磁盘，使加密磁盘不能够进行读写
    输入参数
                device: 设备分区例如：/dev/sdb1
        password： 密码
        passwortlen： 密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_close_encrypted_device(char *device);

    /* 
更改密钥
    输入参数
                device: 设备分区例如：/dev/sdb1
        password： 旧密码
        passwortlen： 旧密码长度
        newpassword： 新密码
        newpasswortlen：新密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_change_password(char *device, char *password, size_t passwortlen, char *newpassword, size_t newpasswortlen);

    /* 
密码重置
    输入参数
                device: 设备分区例如：/dev/sdb1
        FilePath： 恢复密钥文件路径
        newpassword： 新密码
        newpasswortlen：新密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_reset_password(char *device, char *FilePath, char *newpassword, size_t newpasswordlen);

    /* 
判断磁盘是否解锁
    输入参数
                device: 设备分区例如：分区uuid或者/dev/sdb1
    返回值
        0 表示已经解锁
        1 表示未解锁
        其它表示错误
*/
    int crypt_disk_is_opened(char *device);

    /* 
判断磁盘是否可以保留数据加密
   输入参数
                device: 设备分区例如：/dev/sdb1
    返回值
        0 表示可以保留数据加密
        1 表示不可以保留数据加密
        其它表示错误
*/
    int crypt_disk_can_not_formatted(char *device);

    /* 
判断磁盘是否加密
    输入参数
                device: 设备分区例如：/dev/sdb1
    返回值
        0 表示已经加密
        1 表示未加密
        其它表示错误
*/
    int crypt_disk_is_encrypted(char *device);

    /* 
导出恢复密钥
    输入参数
                device: 设备分区例如：/dev/sdb1
                path:备份密钥文件路径
        oldpassword： 旧密码
        oldpasswortlen：旧密码长度
    返回值
        成功返回0
        失败返回非0
*/
    int crypt_disk_export_backup_key(char *device, char *path, char *oldpassword, size_t oldpasswordlen);

    /*
输入参数：device 设备路径
返回参数：path设备默认的备份密钥路径，需传入大于128字节的字符数组，例如char path[129]={0};
返回值：
        0 表示成功
       其它表示错误
*/
    int crypt_disk_default_backup_path(char *device, char *path);

    /*
返回进度值,支持保留数据加密和停用加密
输入参数：无
返回值：
	float类型的小于等于1的值
*/
    float crypt_get_progress_bar(char *device);
    /* 
导出口令
输入参数
		path: 恢复密钥文件
输出参数
	    password：口令
	    passwdLen：口令长度
返回值
        0 表示成功
        其它表示错误,具体参照错误列表
*/
    int crypt_disk_export_password(char *FilePath, char *password, int *passwdLen);

/* 
获取挂载点
输入参数
		volName: 设备路径，例如/dev/sdb1
输出参数
	    mountpoint：挂载路径，传入大于128字节的字符数组，例如char mountpoing[129]={0};
返回值
        0 表示获取挂载点成功，1表示未挂载
        其它表示错误,具体参照错误列表
*/
    int crypt_get_mountpoint(char *volName, char *mountpoint);

/* 
返回占用文件列表
输入参数
		devpath: 设备挂载点
输出参数
	    文件的个数
返回值
        文件列表，NULL表示获取失败
示例：
    int ret = 0;
	char **filelist;
	filelist = get_all_file_status("/media/jxs/", &ret);
	for (int i= 0; i<ret; i++)
	{
		printf("filelist[%d]: %s\n", i, filelist[i]);
	}

	for (int j= 0; j<ret; j++)
	{
		free(filelist[j]);
	}
	free(filelist);
*/
    char **crypt_get_all_file_status(char *devpath, int *count);

/* 
返回设备是否正在使用磁盘加密工具
输入参数
		devpath: 设备名称
返回值
        -1代表函数报错
        0代表当前设备正在使用磁盘加密工具
        1代表当前设备未使用磁盘加密工具
*/
    int check_device_is_using_diskenc(char *devpath);

/* 
返回磁盘加密服务是否处于active状态
输入参数
        无
返回值
        -1代表函数报错
        0代表磁盘加密服务处于未激活状态
        1代表磁盘加密服务处于激活状态
*/
    int check_service_is_active();

/* 
返回各分区是否被磁盘加密服务占用
输入参数
        无
返回值
        -1代表函数报错
        0代表存在分区被磁盘加密服务占用
        1代表不存在分区被磁盘加密服务占用
*/
    int check_all_partition_is_occupied();

/* 
获取映射点
输入参数
		device: 设备路径，例如/dev/sdb1
输出参数
	    path：映射路径，需传入大于128字节的字符数组，例如char path[129]={0};
返回值
        0 表示获取映射点成功，其它表示错误,具体参照错误列表
*/
    int crypt_disk_get_dev_mapper(char *device, char *path);
  

    /*
输入参数：mount_point 挂载点文件路径
        pw_uid：用户id
        pw_gid :组id
返回值：
        0 表示成功
       其它表示错误
*/
    int crypt_disk_change_chown_mountpoint(char *mount_point, uid_t pw_uid, gid_t pw_gid);

    /* 
polkit认证
输入参数
        actid: polkit动作id
输出参数
        无
返回值
        1 认证成功， 0 认证失败
*/ 
    int crypt_disk_authority_check_by_polkit(char *actid);
    /* 
释放共享内存
输入参数
        无
输出参数
        无
返回值
        -1 释放失败， 0 释放成功
*/    
    int delate_shm_for_bar(char *device);
    /*
  是否需要修复磁盘
      输入参数
          device: 设备分区例如：/dev/sdb1
      返回值
          不需要修复返回0
          因为格式化加密中断需要修复返回-1
          因为非格式化加密中断需要修复返回-2
 */
    int crypt_get_disk_reencrypt_status(char *device);

    /*
         非格式化加密中断修复
             输入参数
                 device: 设备分区例如：/dev/sdb1
                 password 密码
                 passwordLen 密码长度
             返回值
                 成功返回0
                 密码不正确返回：KY_PASSPHRASE_ERROR（118）
                 修复失败返回 -1
    */
    int crypt_disk_reencrypt_repair(char *device, char *password, size_t passwordLen, char *path);

    /*
       验证磁盘密码
       输入参数
            device: 设备分区例如：/dev/sdb1
            password 密码
            passwordLen 密码长度
        返回值
            成功返回0
            密码不正确返回：KY_PASSPHRASE_ERROR（118）

*/
    int crypt_disk_verify_device_password(char *device, char *password, size_t passwordLen);
    /*
            获取磁盘UUID
        输入参数：device 设备路径
        返回参数：diskUUID设备映射点
        返回值：
            0 表示成功
            其它表示错误
    */
    int crypt_on_handle_disk_get_device_uuid(char *device, char *diskUUID);
#ifdef __cplusplus
}
#endif
#endif
